! ############################################################
!            Satellite Data Simulation Unit v2
!                     -  param_set_bmcv -
!
! User-defined parameters for the beam convolution routine
! are set up in this subroutine.
!
!              Updated from param_set-BMCV.f by H. M. (May, 2009)
!
 Subroutine param_set_BMCV
   Use MAINparameters
   Use BMCVparameters
   Implicit NONE
!
! 1. Microwave radiometer -------------------------------------------
!
! / The If statement added for SDSU v2.1.2 / 
!
   If ( microw ) then
!
! 1.2 Sensor field of views in [km]*
!    (*: the unit of FOV can be arbitrarily defined but must be shared
!        by "delta_x" and "delta_y".)
!
   Allocate ( fov_ct_microw(mxfreq_microw), fov_dt_microw(mxfreq_microw) )
!
!  Cross-track (x-direction) FOV sizes
!
   fov_ct_microw(1) = 36.
   fov_ct_microw(2) = 18.
   fov_ct_microw(3) = 17.
   fov_ct_microw(4) = 9.7
   fov_ct_microw(5) = 4.2
!
!  Down-track (y-direction) FOV sizes
!
   fov_dt_microw(1) = 60.
   fov_dt_microw(2) = 30.
   fov_dt_microw(3) = 27.
   fov_dt_microw(4) = 16.
   fov_dt_microw(5) = 6.8
!
   End if
!
! 2. Radar ---------------------------------------------------------
!
! / The If statement added for SDSU v2.1.2 / 
!
   If ( radar ) then
!
! 2.2 Sensor field of view in [km]*
!    (*: the unit of FOV can be arbitrarily defined but must be shared
!        by "delta_x" and "delta_y".)
!
   Allocate ( fov_ct_radar(mxfreq_radar), fov_dt_radar(mxfreq_radar) )
!
!  Cross-track (x-direction) FOV sizes
!
   fov_ct_radar(1) = 4.3
   fov_ct_radar(2) = 4.3
   fov_ct_radar(3) = 1.4
!
!  Down-track FOV (y-direction) sizes
!
   fov_dt_radar(1) = 4.3
   fov_dt_radar(2) = 4.3
   fov_dt_radar(3) = 1.7
!
   End if
!
! 3. Visible and IR radiometer -------------------------------------
!
! / The If statement added for SDSU v2.1.2 / 
!
   If ( visIR ) then
!
! 3.2 Sensor field of view in [km]*
!    (*: the unit of FOV can be arbitrarily defined but must be shared
!        by "delta_x" and "delta_y".)
!
   Allocate ( fov_ct_visir(mxwavel), fov_dt_visir(mxwavel) )
!
!  Cross-track (x-direction) FOV sizes
!
   fov_ct_visir(1) = 2.1
   fov_ct_visir(2) = 2.1
   fov_ct_visir(3) = 2.1
   fov_ct_visir(4) = 2.1
   fov_ct_visir(5) = 2.1
!
!  Down-track FOV (y-direction) size
!
   fov_dt_visir(1) = 2.1
   fov_dt_visir(2) = 2.1
   fov_dt_visir(3) = 2.1
   fov_dt_visir(4) = 2.1
   fov_dt_visir(5) = 2.1
!
   Endif
!
   Return
 End Subroutine param_set_BMCV

