! ############################################################
!            Satellite Data Simulation Unit v2
!                     - eddington_pp -
!
! The C.K.'s Eddington RT solver for a plane-parallel atmosphere
!
!                 Updated from eddington_pp.f by H. M. (May, 2009)
!
 subroutine eddington_pp
   Use MAINparameters
   Use MWRparameters
!
!     CHRIS KUMMEROW
!     INCLUDES ASYMPTOTIC EXPRESSIONS FOR TERM3, TERM4, AND TERM5 IN
!     THE LIMIT OF SMALL EFFECTIVE OPTICAL DEPTHS; BILL OLSON FEB, 1995.
!
   real, dimension(:), allocatable ::  KEXT, SALB, ASYM
   real, dimension(:), allocatable ::  L, H, B0, B1
   real, dimension(:), allocatable ::  BB, DP, DM
   real, dimension(:), allocatable ::  Z, IOUT, dumR
   real, dimension(:,:), allocatable ::  W, I_IN
   integer, dimension(:), allocatable :: dumI
   real :: MU, NU
   real, parameter :: FISOT = 2.7
!
   allocate ( KEXT(nlyr), SALB(nlyr), ASYM(nlyr) )
   allocate ( L(nlyr), H(nlyr), B0(nlyr), B1(nlyr) )
   allocate ( W(2*nlyr,2*nlyr), BB(2*nlyr), DP(nlyr), DM(nlyr) )
   allocate ( dumR(2*nlyr), dumI(2*nlyr) )
   allocate ( Z(0:nlyr), IOUT(0:nlyr), I_IN(nlyr+1,100) )
!
!     Plane parallel Eddington computes Tb for one vertical hydrometeor
!     column at a time.  The code therefore loops over nx & ny
!
   do nx = 1, ngridx
      do ny = 1, ngridy
!
         Z(0) = hght_lev(nx,ny,0)
         BTEMP = tskin(nx,ny)
         DO J = 1,NLYR
            Z(J)  = hght_lev(nx,ny,J)
            B0(J) = temp_lev(nx,ny,J-1)
            B1(J) = (temp_lev(nx,ny,J) - temp_lev(nx,ny,J-1))/(Z(J) - Z(J-1))
            KEXT(J) = kexttot(nx,ny,J)
            SALB(J) = salbtot(nx,ny,J)
            ASYM(J) = asymtot(nx,ny,J)
            L(J) = SQRT(3.*KEXT(J)*KEXT(J)*(1. - SALB(J))* &
                                  (1. - SALB(J)*ASYM(J)))
            H(J) = 1.5*KEXT(J)*(1. - SALB(J)*ASYM(J))
         END DO
!
!         FILL IN THE MATRIX ELEMENTS WHICH FORM THE BOUNDARY CONDITIONS
!         AT THE TOP, BOTTOM AND LAYER INTERFACES OF THE CLOUD.  THERE ARE
!         TWO QUANTITIES, D+ "DP", AND D- "DM" AT EACH BOUNDARY, SO THE 
!         MATRIX HAS DIMENSIONS  2*NLYR BY 2*NLYR.
!         ORDER OF D'S:  D+(1),D-(1),D+(2),D-(2), .... , D+(NLYR),D-(NLYR)
!
!         SET ALL MATRIX ELEMENTS TO ZERO
!
         W = 0.0
!
!         FILL IN THE NON-ZERO MATRIX ELEMENTS
!
         W(1,1)   = ((EBAR(nx,ny) - 2.)*L(1)/H(1)) + EBAR(nx,ny)
         W(1,2)   = ((2. - EBAR(nx,ny))*L(1)/H(1)) + EBAR(nx,ny)
         DO I = 2,2*(NLYR-1),2
            W(I,I-1)   =  (1. - L(I/2)/H(I/2))*EXP(+L(I/2)*(Z(I/2)- &
                          Z(I/2-1)))
            W(I,I  )   =  (1. + L(I/2)/H(I/2))*EXP(-L(I/2)*(Z(I/2)- &
                          Z(I/2-1)))
            W(I,I+1)   = -(1. - L(I/2+1)/H(I/2+1))
            W(I,I+2)   = -(1. + L(I/2+1)/H(I/2+1))

            W(I+1,I-1) =  (1. + L(I/2)/H(I/2))*EXP(+L(I/2)*(Z(I/2)- &
                          Z(I/2-1)))
            W(I+1,I)   =  (1. - L(I/2)/H(I/2))*EXP(-L(I/2)*(Z(I/2)- &
                          Z(I/2-1)))
            W(I+1,I+1) = -(1. + L(I/2+1)/H(I/2+1))
            W(I+1,I+2) = -(1. - L(I/2+1)/H(I/2+1))
         END DO
         W(2*NLYR,2*NLYR-1) =  (1. + L(NLYR)/H(NLYR))*EXP(+L(NLYR)* &
                               (Z(NLYR)-Z(NLYR-1)))
         W(2*NLYR,2*NLYR)   =  (1. - L(NLYR)/H(NLYR))*EXP(-L(NLYR)* &
                               (Z(NLYR)-Z(NLYR-1)))
!
!   FILL IN THE ROW OF CONSTANTS IN THE LINEAR EQUATIONS
!
         BB(1)    = EBAR(nx,ny) *BTEMP - EBAR(nx,ny)*B0(1) - &
                   (EBAR(nx,ny) - 2.)*B1(1)/H(1)
         DO I = 2,2*(NLYR-1),2
            BB(I)   =  + B1(I/2)/H(I/2) - B1(I/2+1)/H(I/2+1)
            BB(I+1) =  - B1(I/2)/H(I/2) + B1(I/2+1)/H(I/2+1)
         END DO
         BB(2*NLYR)  =  FISOT - B0(NLYR) - B1(NLYR)*(Z(NLYR) - &
                        Z(NLYR-1) + 1/H(NLYR))
!
!         MATRIX INVERSION IN DONE IN SUBROUTINE LINPAK
!
         CALL LINPAK(nlyr, W, BB, dumR, dumI, RCOND)
!
         DO I = 1,NLYR
            DP(I) = BB(2*I-1)
            DM(I) = BB(2*I)
         END DO

!         AFTER D'S ARE KNOWN, CALCULATE SURFACE RADIANCE

         MU = UMU
         NU = -UMU

!         FOR THE FOLLOWING CALCULATIONS, REFER TO APPENDIX B OF THESIS
!         *************************************************************
      
         IF ( LAMBERT ) THEN 
!           CALCULATE THE DOWNWELLING FLUX AT 81 ANGLES
            NANG = 81
            DO NN = 1,NANG
               XNU = -(2.*NN - 1.)/(NANG*2.)
               I_IN(NLYR+1,NN) = FISOT
!             LOOP THROUGH THE REMAINING LAYERS
               DO J = NLYR,1,-1
!               CALCULATE RADIANCE FROM TOP OF LAYER "J"
                  XA = B0(J) - 1.5*SALB(J)*ASYM(J)*XNU*B1(J)/H(J)
                  XB = B1(J)
                  XC = SALB(J)*DP(J)*(1. - 1.5*ASYM(J)*XNU*L(J)/H(J))
                  XD = SALB(J)*DM(J)*(1. + 1.5*ASYM(J)*XNU*L(J)/H(J))
                  YA = KEXT(J)/XNU
                  YB = YA + L(J)
                  YC = YA - L(J)
                  DZ = Z(J) - Z(J-1)

                  TERM1 = I_IN(J+1,NN)*EXP(YA*DZ)
                  TERM2 = XA*(1. - EXP(YA*DZ))
                  IF(ABS(ya*dz) .LT. 1.e-5) THEN
                     TERM3=-XB*YA*DZ*DZ
                  ELSE
                     TERM3 = XB/YA*(EXP(YA*DZ)*(1. - YA*DZ) - 1.)
                  ENDIF
                  if(abs(yb*dz) .lt. 1.e-5) then
                     term4=-xc*ya*dz
                  else
                     TERM4 = XC*YA/YB*(1. - EXP(YB*DZ))
                  end if
                  if(abs(yc*dz) .lt. 1.e-5) then
                     term5=-xd*ya*dz
                  else
                     TERM5 = XD*YA/YC*(1. - EXP(YC*DZ))
                  end if
                  I_IN(J,NN) = TERM1 + TERM2 + TERM3 + TERM4 + TERM5
               END DO
            END DO
!
!           CALCULATE THE TOTAL DOWNWELLING FLUX REACHING THE SURFACE
!
            XIUP = 0.
            DO NN = 1,NANG
               XIUP = XIUP + I_IN(1,NN)*(1./NANG)*(2.*NN-1.)/(2.*NANG)
            END DO
            XIUP = 2.*XIUP
!          
         ELSE
!
!           CALCULATE THE DOWNWELLING FLUX AT ANGLE MU ONLY
!
            NN = 22            ! THIS IS A DUMMY INDEX FOR I_IN
            I_IN(NLYR+1,NN) = FISOT
!           LOOP THROUGH THE REMAINING LAYERS
            DO J = NLYR,1,-1
!             CALCULATE RADIANCE FROM TOP OF LAYER "J"
               XA = B0(J) - 1.5*SALB(J)*ASYM(J)*NU*B1(J)/H(J)
               XB = B1(J)
               XC = SALB(J)*DP(J)*(1. - 1.5*ASYM(J)*NU*L(J)/H(J))
               XD = SALB(J)*DM(J)*(1. + 1.5*ASYM(J)*NU*L(J)/H(J))
               YA = KEXT(J)/NU
               YB = YA + L(J)
               YC = YA - L(J)
               DZ = Z(J) - Z(J-1)

               TERM1 = I_IN(J+1,NN)*EXP(YA*DZ)
               TERM2 = XA*(1. - EXP(YA*DZ))
               if(abs(ya*dz) .lt. 1.e-5) then
                  term3=-xb*ya*dz*dz
               else
                  TERM3 = XB/YA*(EXP(YA*DZ)*(1. - YA*DZ) - 1.)
               end if
               if(abs(yb*dz) .lt. 1.e-5) then
                  term4=-xc*ya*dz
               else
                  TERM4 = XC*YA/YB*(1. - EXP(YB*DZ))
               end if
               if(abs(yc*dz) .lt. 1.e-5) then
                  term5=-xd*ya*dz
               else
                  TERM5 = XD*YA/YC*(1. - EXP(YC*DZ))
               end if
               I_IN(J,NN) = TERM1 + TERM2 + TERM3 + TERM4 + TERM5
            END DO
            XIUP = I_IN(1,22)
!
          ENDIF
!
          IOUT(0) = EMIS(nx,ny)*BTEMP + (1. - EMIS(nx,ny))*XIUP
          DO J = 1,NLYR
!           CALCULATE THE UPWELLING RADIANCES AT THE TOP OF EACH LAYER J
             XA = B0(J) - 1.5*SALB(J)*ASYM(J)*MU*B1(J)/H(J)
             XB = B1(J)
             XC = SALB(J)*DP(J)*(1. - 1.5*ASYM(J)*MU*L(J)/H(J))               
             XD = SALB(J)*DM(J)*(1. + 1.5*ASYM(J)*MU*L(J)/H(J))
             YA = KEXT(J)/MU
             YB = YA + L(J)
             YC = YA - L(J)
             DZ = Z(J) - Z(J-1)
             TERM1 = IOUT(J-1)*EXP(-YA*DZ)
             TERM2 = XA*(1. - EXP(-YA*DZ))
             if(abs(ya*dz) .lt. 1.e-5) then
                term3=0.
             else
                TERM3 = XB/YA*(YA*DZ - 1. + EXP(-YA*DZ))
             end if
             if(abs(yb*dz) .lt. 1.e-5) then
                term4=xc*ya*dz*exp(-ya*dz)
             else
                TERM4 = XC*YA/YB*(EXP( (YB-YA)*DZ ) - EXP(-YA*DZ) )
             end if
             if(abs(yc*dz) .lt. 1.e-5) then
                term5=xd*ya*dz*exp(-ya*dz)
             else
                TERM5 = XD*YA/YC*EXP(-YA*DZ)*(EXP(YC*DZ) - 1.)
             end if
             IOUT(J) = TERM1 + TERM2 + TERM3 + TERM4 + TERM5
          END DO
!
          TB(nx,ny) = IOUT(NLYR)
!
       end do
    end do
    return
  end subroutine eddington_pp
