subroutine abgask(iug,nln,nch,dintvl,nww,wlb,dp,np,amtpb,t,tau,err)
! calculate absorption coefficients.
!--- history
! 06.03.17 created (miho)
! 07.11.30 fix dimension (nln => knln, nch => kch) 
! 08.08.14 add dintvl
! 08. 9.26 modified Fortran 90 free-style form
! 08.12.22 bug fixed.
!--- input
! iug      i     read unit number of gas line absorption para file
! nln      i     number of layers
! wla      r     the lower boundary of wavelength
! wlb      r     the upper boundary of wavelength
! dp     r(nln)  residual of the pressure   
! np     i(nln)  grid number of the pressure
! t      r(nln)  temperature of the layer
!--- output
! tau  r(nln,kch,kww) optical thickness
!---
  use VIRparameters, only: kmol,kp,kt,kmol1,kww,knln,kch
  implicit none
! input
! / knln has been replaced by nln for SDSU /
!
  integer,intent(in):: iug       !! device number
  integer,intent(in):: nln       !! device number
  integer,intent(in):: nch       !! device number
  integer,intent(in):: dintvl    !! device number
  integer,intent(in):: nww       !! device number
  real(8),intent(in):: wlb       !! device number
  real(8),intent(in):: dp(knln)  !! device number
  integer,intent(in):: np(knln)  !! device number
  real(8),intent(in):: amtpb(kmol1,knln) !! gas concentration
  real(8),intent(in):: t(knln)   !! temperature in layer

! output
  real(8),intent(out):: tau(knln,kch,kww) !! optical thickness
  character,intent(inout):: err*64

! work
  integer:: irec,iww,l,ich,imol,it
  real(4):: ak(kch,kp,kt,kmol1)
  real(8):: akt(kt,knln),knu(knln)
!--- exec
  tau(1:nln,1:nch,1:nww)=0.d0

! find the position of the beginning of the data.
! wlb = 10+10**(irec/dintvl)
  irec=int((log10(1.d4/wlb)-1.d0)*dintvl)
  do iww=1,nww
     irec=irec+1
     read(iug,rec=irec) ak(1:nch,1:kp,1:kt,1:kmol1)
     if(irec.lt.1) then
        err='wavenumber out of the range [0.2-1000 micron]'
        return
     endif

     do ich=1,nch
        do imol=1,kmol1
           do l=1,nln
              do it=1,kt
                 akt(it,l)=ak(ich,np(l)-1,it,imol) &
                      +(ak(ich,np(l),it,imol)-ak(ich,np(l)-1,it,imol))*dp(l)
              enddo
           enddo
           call tdok2(nln,akt,t,knu)
           tau(1:nln,ich,iww)=tau(1:nln,ich,iww)+knu(1:nln)*amtpb(imol,1:nln)
        enddo
     enddo
  enddo
  return
end subroutine abgask
subroutine sdp2(nln,p,dp,np)
! derive absorption coefficient from pt table and fitting.
!--history
! 02.02.27  Modified knupt in ckdng.f (Zhang Hua)
! 02.11.25  Modified from knpt1.      (Miho Sekiguchi)
! 03.09.09  Modified from sdp for free source form.(Miho Sekiguchi)
! 07.03.28  Modified about dimension nln => knln. (Miho Sekiguchi)
! 08. 9.25 modified Fortran 90 free-style form
!--
  use VIRparameters, only: kp,knln
  implicit none

! input & output
  integer,intent(in):: nln       !! number of layer
  real(8),intent(in):: p(knln)   !! pressure in layer
  real(8),intent(out):: dp(knln) !! ratio of pressure grid
  integer,intent(out):: np(knln) !! number of pressure grid

! works
  integer:: l,ip
  real(8):: prs(1:26)=(/ &
       1.0130d+3,  0.63096d+3, 0.39811d+3,  0.25119d+3,  0.15849d+3, &
       1.0000d+2,  0.63096d+2, 0.39811d+2,  0.25119d+2,  0.15849d+2, &
       1.0000d+1,  0.63096d+1, 0.39811d+1,  0.25119d+1,  0.15849d+1, &
       1.0000d+0,  0.63096d+0, 0.39811d+0,  0.25119d+0,  0.15849d+0, &
       1.0000d-1,  0.63096d-1, 0.39811d-1,  0.25119d-1,  0.15849d-1, &
       1.0000d-2/)               !! pressure grid
!--exec
  do l=1,nln
     if(p(l) < prs(kp)) then
        np(l)=kp
        dp(l)=1.d0
        cycle
     endif
     do ip = 1, kp
        if (p(l) >= prs(ip)) exit
     enddo
     np(l)=ip
     if(np(l) > kp) np(l)=kp
     if(np(l) <= 1) np(l)=2
     dp(l)=log10(p(l)/prs(np(l)-1))/log10(prs(np(l))/prs(np(l)-1))
  enddo
end subroutine sdp2
subroutine tdok2(nln,akt,t,knu)
!----------------------------------------------------------------------c
!                using Shi's formula: (t/to)**(a+bt)
!----------------------------------------------------------------------c
  use VIRparameters, only: kt,knln
  implicit none
! / Correction: the dimension size 'nln' -> 'knln' /
  integer,intent(in):: nln       !! number of layer
  real(8),intent(inout):: akt(kt,knln)   !! absorption coefficients at T grid
  real(8),intent(in):: t(knln)   !! temperature in layer
  real(8),intent(out):: knu(knln) !! absorption coefficients
! works
  integer:: l
  real(8):: a,b,at1,at2,bt1,bt2
!--exec
  at1 = log10(200.d0/260.d0); at2 = log10(320.d0/260.d0)
  
  do l=1,nln
     if(akt(2,l) <= -14.d0.and.akt(1,l) > -14.d0.and. &
          akt(3,l) > -14.d0) akt(2,l)=(akt(1,l)+akt(3,l))*0.5d0
     bt1 = akt(1,l)-akt(2,l)
     bt2 = akt(3,l)-akt(2,l)
     if(bt1 == 0.d0 .and. bt2 == 0.d0) then
        knu(l)=10**akt(2,l); cycle
     endif
     b   = (bt2/at2-bt1/at1)/120.d0
     a   = bt2/at2 - b*320.d0
     
     knu(l)=10.d0**(akt(2,l))*(t(l)/260.d0)**(a+b*t(l))
  enddo
  return
end subroutine tdok2
